# -*- coding: utf-8 -*-
import logging
from django.db import transaction
from client import NfsaasClient
from models import EnvironmentAttr, PlanAttr, HostAttr

LOG = logging.getLogger(__name__)


class NfsaasProvider(object):

    @classmethod
    def get_credentials(self, environment):
        LOG.info("Getting credentials...")
        from dbaas_credentials.credential import Credential
        from dbaas_credentials.models import CredentialType
        integration = CredentialType.objects.get(type=CredentialType.NFSAAS)

        return Credential.get_credentials(environment=environment, integration=integration)

    @classmethod
    def auth(self, environment):
        LOG.info("Conecting with nfsaas...")
        credentials = self.get_credentials(environment=environment)
        return NfsaasClient(baseurl=credentials.endpoint,
                            teamid=credentials.team,
                            projectid=credentials.project,
                            username=credentials.user,
                            password=credentials.password)

    @classmethod
    @transaction.commit_on_success
    def grant_access(self, environment, plan, host, export_id):

        LOG.info("Creating access!")

        nfsaas = self.auth(environment=environment)
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment

        access = nfsaas.create_access(environmentid=nfsaas_environmentid,
                                      sizeid=nfsaas_planid,
                                      exportid=export_id,
                                      host=host.address)

        LOG.info("Access created: %s" % access)

    @classmethod
    @transaction.commit_on_success
    def revoke_access(self, environment, plan, host, export_id):

        LOG.info("Removing access on export (id=%s) from host %s" % (export_id, host))
        nfsaas = self.auth(environment=environment)
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan

        accesses = nfsaas.list_access(environmentid=nfsaas_environmentid,
                                      sizeid=nfsaas_planid,
                                      exportid=export_id)

        for access in accesses:
            host_nfs = access["hosts"]
            host_network = nfsaas.change_ip(host.address)
            if host_nfs == host_network:
                LOG.info("Removing access on export (id=%s) from host %s" % (export_id, host))
                nfsaas.drop_access(environmentid=nfsaas_environmentid,
                                   sizeid=nfsaas_planid,
                                   exportid=export_id,
                                   accessid=access['id'])
                LOG.info("Access deleted: %s" % access)
                break

    @classmethod
    @transaction.commit_on_success
    def create_disk(self, environment, plan, host):

        nfsaas = self.auth(environment=environment)
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment

        LOG.info("Creating export on environmen %s and size %s" % (nfsaas_environmentid, nfsaas_planid))
        export = nfsaas.create_export(environmentid=nfsaas_environmentid, sizeid=nfsaas_planid)
        LOG.info("Export created: %s" % export)

        self.grant_access(environment, plan, host, export['id'])

        LOG.info("Saving export info on nfsaas host attr")
        hostattr = HostAttr(host=host, nfsaas_export_id=export['id'], nfsaas_path=export['path'])
        hostattr.save()

        return export

    @classmethod
    @transaction.commit_on_success
    def destroy_disk(self, environment, plan, host):

        if not HostAttr.objects.filter(host=host).exists():
            LOG.info("There is no HostAttr for this host %s. It may be an arbiter." % (host))
            return True

        nfsaas = self.auth(environment=environment)

        hostattr = HostAttr.objects.get(host=host)
        export_id = hostattr.nfsaas_export_id
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan

        accesses = nfsaas.list_access(environmentid=nfsaas_environmentid, sizeid=nfsaas_planid, exportid=export_id)

        for access in accesses:
            LOG.info("Removing access on export (id=%s) from host %s" % (export_id, host))
            nfsaas.drop_access(environmentid=nfsaas_environmentid,
                               sizeid=nfsaas_planid,
                               exportid=export_id,
                               accessid=access['id'])
            LOG.info("Access deleted: %s" % access)

        LOG.info("Deleting register from nfsaas host attr")
        hostattr.delete()

        try:
            LOG.info("Env: %s, size: %s, export: %s" % (nfsaas_environmentid, nfsaas_planid, export_id))
            deleted_export = nfsaas.drop_export(environmentid=nfsaas_environmentid, sizeid=nfsaas_planid, exportid=export_id)
            LOG.info("Export deleted: %s" % deleted_export)
            return deleted_export
        except Exception, e:
            print e
            return None

    @classmethod
    def create_snapshot(self, environment, plan, host):
        nfsaas = self.auth(environment=environment)

        hostattr = HostAttr.objects.get(host=host)
        export_id = hostattr.nfsaas_export_id
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan

        snapshot = nfsaas.create_snapshot(environmentid=nfsaas_environmentid, sizeid=nfsaas_planid, exportid=export_id)

        return snapshot

    @classmethod
    def remove_snapshot(self, environment, plan, host, snapshopt):

        hostattr = HostAttr.objects.get(host=host)
        export_id = hostattr.nfsaas_export_id
        nfsaas_environmentid = EnvironmentAttr.objects.get(dbaas_environment=environment).nfsaas_environment
        nfsaas_planid = PlanAttr.objects.get(dbaas_plan=plan).nfsaas_plan
        nfsaas = self.auth(environment=environment)
        nfsaas.drop_snapshot(environmentid=nfsaas_environmentid,
                             sizeid=nfsaas_planid,
                             exportid=export_id,
                             snapshotid=snapshopt)



# -*- coding: utf-8 -*-
import urllib2
import json


class NfsaasClient(object):

    def __init__(self, baseurl, teamid, projectid, username, password):

        self.base_url = baseurl
        self.teamid = teamid
        self.projectid = projectid
        self.username = username
        self.password = password

        p = urllib2.HTTPPasswordMgrWithDefaultRealm()
        p.add_password(None, self.base_url, self.username, self.password)
        handler = urllib2.HTTPBasicAuthHandler(p)
        opener = urllib2.build_opener(handler)
        urllib2.install_opener(opener)

    def list_teams(self):
        url = '%stimes/' % (self.base_url)
        return json.load(urllib2.urlopen(url))

    def list_projects(self):
        url = '%stimes/%s/projetos/' % (self.base_url, self.teamid)
        return json.load(urllib2.urlopen(url))

    def list_environments(self):
        url = '%stimes/%s/projetos/%s/ambientes/' % (
            self.base_url, self.teamid, self.projectid)
        return json.load(urllib2.urlopen(url))

    def list_sizes(self, environmentid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/' % (
            self.base_url, self.teamid, self.projectid, environmentid)
        return json.load(urllib2.urlopen(url))

    def list_exports(self, environmentid, sizeid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid)
        return json.load(urllib2.urlopen(url))

    def get_export(self, environmentid, sizeid, exportid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid,
            exportid)
        return json.loads(json.load(urllib2.urlopen(url)))

    def create_export(self, environmentid, sizeid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid)
        request = urllib2.Request(url, data="{}")
        request.add_header("Content-Type", "application/json")
        newexport = json.load(urllib2.urlopen(request))
        newexport = json.loads(newexport[0])
        return newexport

    def drop_export(self, environmentid, sizeid, exportid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid)
        request = urllib2.Request(url)
        request.get_method = lambda: 'DELETE'
        deleted_export = urllib2.urlopen(request)
        return deleted_export

    def list_access(self, environmentid, sizeid, exportid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/acessos/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid)
        return json.load(urllib2.urlopen(url))

    def change_ip(self, host):
        h = host.split('.')
        h = h[:len(h) - 1]
        return '.'.join(h) + '.0/24'

    def create_access(self, environmentid, sizeid, exportid, host):
        host = self.change_ip(host)
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/acessos/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid)
        data = """{
            "hosts": "%s",
            "permission": {
                "type": "read-write"
            }
        }""" % (host,)
        request = urllib2.Request(url, data=data)
        request.add_header("Content-Type", "application/json")
        newaccess = json.load(urllib2.urlopen(request))
        newaccess = json.loads(newaccess[0])
        return newaccess

    def drop_access(self, environmentid, sizeid, exportid, accessid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/acessos/%s/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid, accessid)
        request = urllib2.Request(url)
        request.get_method = lambda: 'DELETE'
        deleted_acesso = urllib2.urlopen(request)
        return deleted_acesso

    def list_snapshots(self, environmentid, sizeid, exportid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/snapshots/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid)
        return json.load(urllib2.urlopen(url))

    def get_snapshot(self, environmentid, sizeid, exportid, snapshotid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/snapshots/%s/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid, snapshotid)
        return json.load(urllib2.urlopen(url))

    def create_snapshot(self, environmentid, sizeid, exportid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/snapshots/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid)
        request = urllib2.Request(url, data="{}")
        request.add_header("Content-Type", "application/json")
        return json.load(urllib2.urlopen(request))

    def drop_snapshot(self, environmentid, sizeid, exportid, snapshotid):
        url = '%stimes/%s/projetos/%s/ambientes/%s/tamanhos/%s/exports/%s/snapshots/%s/' % (
            self.base_url, self.teamid, self.projectid, environmentid, sizeid, exportid, snapshotid)
        request = urllib2.Request(url)
        request.get_method = lambda: 'DELETE'
        deleted_export = urllib2.urlopen(request)
        return deleted_export



# -*- coding: utf-8 -*-
from util.models import BaseModel
from django.db import models
from django.utils.translation import ugettext_lazy as _


class EnvironmentAttr(BaseModel):
    dbaas_environment = models.ForeignKey(
        'physical.Environment', related_name="nfsaas_environment_attributes")
    nfsaas_environment = models.CharField(
        verbose_name=_("Environment ID"), max_length=10)

    class Meta:
        verbose_name_plural = 'NFaaS Custom Environment Attributes'


class PlanAttr(BaseModel):
    dbaas_plan = models.ForeignKey(
        'physical.Plan', related_name="nfsaas_plan_attributes")
    nfsaas_plan = models.CharField(verbose_name=_("Plan ID"), max_length=10)

    class Meta:
        verbose_name_plural = 'NFaaS Custom Plan Attributes'


class HostAttr(BaseModel):
    host = models.ForeignKey(
        'physical.Host', related_name="nfsaas_host_attributes")
    nfsaas_export_id = models.CharField(
        verbose_name=_("Export ID"), max_length=10)
    nfsaas_path = models.CharField(verbose_name=_("Path"), max_length=100)
    is_active = models.BooleanField(
        verbose_name=_("Is instance active"), default=True)

    class Meta:
        verbose_name_plural = 'NFaaS Custom Host Attributes'
